%% Approximate the alpha curve and its derivative

function [AA,AA_pr] = alpha_curve(PP,pp,alpha,alpha0)

%% 1.a. Preliminaries
% Defines function inputs and outputs.

% output vector BB
AA = ones(size(PP));
AA_pr = ones(size(PP));

% number of tax thresholds
T = length(pp);

% 
pp_all = [0; pp; 1];
alpha_all = [alpha0; alpha; 1];

% first approximation of the derivative of the alpha curve
%dd = approx_deriv(pp_all,alpha_all);
%dd(1)=-0.005

% epsilon determining the precision of the approximation of the beta(i)
eps = 1e-14;

% time limit for while loops
timelimit = 20;

%% 1.b. Linear interpolation

l0_temp = -1;
temp = alpha_lin(l0_temp,pp_all,alpha_all);

for l0=-1:(0.001):1
    if alpha_lin(l0,pp_all,alpha_all)<temp
        l0_temp = l0;
        temp = alpha_lin(l0,pp_all,alpha_all);
    end
end

[~,dd] = alpha_lin(l0_temp,pp_all,alpha_all);

%pp_draw= linspace(0,1,1000);
%dd_draw= interp1(pp_all,dd,pp_draw,'linear')
%plot(pp_draw,dd_draw)

function [min_l,ll] = alpha_lin(l0,pp_all,alpha_all)
    
    U = ones(length(pp_all)-1,1);
    U(1:2:end)=-1;
    ll = cumsum(2*U.*(alpha_all(2:end)-alpha_all(1:end-1))./(pp_all(2:end)-pp_all(1:end-1)));
    ll(1:2:end)=-ll(1:2:end);
    ll = [0; ll]+l0*[1; U];
    ss = (ll(2:end)-ll(1:end-1))./(pp_all(2:end)-pp_all(1:end-1));
    min_l = sum(abs(ss(2:end)-ss(1:end-1)));
    
end


%% 1.c. Approximation of the upper part (above the first threshold)

% vectors of all points that have been set so far
La = [dd(1)]; % alpha_pr

% browse through the raw brackets to set the intermediary points so that
% the successive integrals of derivative of the alpha curve are equals to
% the alphas

for i=0:T
disp(['Currently estimating the alpha curve within bracket ',num2str(i),'...'])

    % call to the function f_alpha that will compute the values of the
    % derivative of alpha at points of pp0
    [AA,AA_pr,La] = f_alpha(pp_all,La,dd(i+2:end),alpha_all(i+2)-alpha_all(i+1),alpha_all(i+1),PP,AA,AA_pr,eps,timelimit);
    
end
end


function [AA,AA_pr,La] = f_alpha(pp,La,dd,Dalpha,alpha0,PP,AA,AA_pr,eps,timelimit)
    
    j = length(La);
    a_init = dd(1);
    dd = dd(2:end);
    
    % derivative of the alpha curve interpolation
    alpha_pr = @(p,a)(interp1(pp,[La(:); a; dd],p,'pchip','extrap'));
    
    % alpha-integral between pp(j) and pp(j+1)
    int_a = @(a)(quadgk(@(q)(alpha_pr(q,a)),pp(j),pp(j+1),'MaxIntervalCount',50000));
    
    temp = int_a(a_init);
    
    if temp<Dalpha
        a_l = a_init;
        a_h = a_init;
        while int_a(a_h)<Dalpha
            a_h = a_h+1;
        end
    else
        a_l = a_init;
        a_h = a_init;
        while int_a(a_l)>=Dalpha
            a_l = a_l-1;
        end
    end
    
    % adjust a while the integral temp is not close enough to Dalpha
    t = cputime;
    while abs(temp-Dalpha)>eps
        
        a_temp = (a_l+a_h)/2;
        
        if int_a(a_temp)<Dalpha
            a_l = a_temp;
        else
            a_h = a_temp;
        end
        
        temp = int_a(a_temp);
        if cputime-t > timelimit
            break
        end
    end
    
    AA_pr((PP>=pp(j))&(PP<=pp(j+1))) = alpha_pr(PP((PP>=pp(j))&(PP<=pp(j+1))),a_temp);
    mesh = linspace(pp(j),pp(j+1),200);
    AA0 = zeros(200,1);
    for i=1:200
        AA0(i) = quadgk(@(q)(alpha_pr(q,a_temp)),pp(j),mesh(i),'MaxIntervalCount',50000);
    end
    AA((PP>=pp(j))&(PP<=pp(j+1))) = alpha0+qinterp1(mesh,AA0,PP((PP>=pp(j))&(PP<=pp(j+1))));
    
    La = [La(:); a_temp];
end