/********Stata code upgrading urban survey tabulations on the basis of top income tax data*********/


#delimit;
clear;
set more off;

cd "D:\Dropbox\WIDChina\PYZ2016DistributionSeries\";


/************importing upgrade factors*****/
import excel ChinaUrban_ParetoCurves_yf_raw.xlsx, sheet("UpgradeFactors") cellrange(A1:w26) firstrow clear;
replace p=1000*p;
sort p;
save StataFiles/temp/upgradefactors.dta, replace;

/********** upgrading urban gperc series not using yearly upgrade variations****/
use StataFiles/gperc_yf_UrbanChina_raw.dta, clear;
sort p;
merge m:1 p using StataFiles/temp/upgradefactors.dta, nogen;
sort year p;
replace ythr=ythr*upythr if p>=80000; 
replace yint=yint*upyint if p>=80000; 
replace ytop=ytop*upyint if p==99500;
gsort year -p;
replace ytop=(f*yint+(100000-p[_n-1])*ytop[_n-1])/(100000-p) if p<99500;
replace b=ytop/ythr;
replace ythr=. if p>99500;
replace yint=. if p>99500;
replace ytop=. if p>99500;
replace b=. if p>99500;
drop u*;
sort year p;
/***consistency check*/
/*gen check=0;
replace check=1 if yint<ythr | (yint>ythr[_n+1] & p<99999);
/*no consistency pb at all*/
ta check;
drop check;*/
save StataFiles/gperc_yf_UrbanChina.dta, replace;
replace p=p/100000;
rename ythr thr;
sort year p;
keep year thr p b;
order year thr p b;
save StataFiles/temp/upgradegperc.dta, replace;


/*****exporting gperc up to p=0.995 in rawdata xls format for MatLab simulations above p=0.995****/
forvalues year=1978/2015 
{;
/****exporting gperc thresholds and inverted Pareto coeff*/
use StataFiles/temp/upgradegperc.dta, clear;
keep if year==`year';
drop year;
drop if p>0.995;
export excel using Tabulations/rawdata_bysheet_urbanChina_corr.xls, sheet("`year'") sheetmodify cell(a3) firstrow(variables);
/*****exporting year and average on top of tabulation*/
keep if p==0 | p==0.01;
gen v1="year";
replace v1="average" if p==0.01;
gen v2=`year' if p==0;
replace v2=b[_n-1]*thr[_n-1] if p==0.01;
keep v1 v2;
export excel using Tabulations/rawdata_bysheet_urbanChina_corr.xls, sheet("`year'") sheetmodify cell(a1);
/*****setting thr=0 and b=. for p=0*/
drop if _n==2;
drop v1;
gen v1=0;
drop v2;
gen v2=0;
gen v3=.;
export excel using Tabulations/rawdata_bysheet_urbanChina_corr.xls, sheet("`year'") sheetmodify cell(a4);
};




